package de.hdm_stuttgart.battlearena.Controller;

import de.hdm_stuttgart.battlearena.Controller.Utilities.CreateMediaPlayer;
import de.hdm_stuttgart.battlearena.Controller.Utilities.MusicPlayerSingleton;
import de.hdm_stuttgart.battlearena.Controller.Utilities.SceneLoader;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.geometry.Rectangle2D;
import javafx.scene.control.Button;
import javafx.scene.image.ImageView;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.StackPane;
import javafx.scene.media.Media;
import javafx.scene.media.MediaPlayer;
import javafx.scene.media.MediaView;
import javafx.stage.Screen;

import java.io.File;
import java.net.URL;
import java.util.*;

public class MenuBorderPaneController implements Initializable {
    @FXML
    private BorderPane parent;
    @FXML
    private Button btnRight, btnLeft;
    @FXML
    private ImageView imgLeft, imgRight;
    @FXML
    private StackPane center;
    private int musicCounter = 1, videoCounter = 0;
    private final SceneLoader sceneLoader = new SceneLoader();
    private MediaPlayer musicPlayer, mediaPlayer;
    private MediaView mediaView;
    private final String[] videoFiles = {"depression", "allMyFellas", "wooOOoo", "myMind", "dogCheese", "gta", "cat", "bobama", "roomba", "firework", "cheezburger",
            "kangaroo", "lifeCouldBeMonke", "seal", "imNotYou", "parkingTickets", "russianKid", "rejectHumanity", "horse", "catSitting", "pablo", "holyCrap",
            "lessGoo", "sadCat", "basketball", "yoinkySploinky", "msPuff", "=D", "banana", "chaCha", "async", "sadHorse", "minecraftCat", "muecke", "top10Cats", "dog",
            "pot", "mineCraftCat2", "catEating", "bear", "pancake", "frog", "gtfo", "carl", "dog2", "slippery", "wolf", "legCat", "sad", "waaahhh"};

    List<String> shuffledVideos = Arrays.asList(videoFiles);

    Rectangle2D screen = Screen.getPrimary().getVisualBounds();
    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        center.getChildren().add(sceneLoader.loadScene("MainMenu"));
//        set size for rusty gear image depending on screen resolution
        final double imageWidth = screen.getWidth() / 6;
        imgRight.setFitWidth(imageWidth);
        imgRight.setPreserveRatio(true);
        imgLeft.setFitWidth(imageWidth);
        imgLeft.setPreserveRatio(true);

        backgroundMusic("cocBackgroundMusicTest.mp3");
        easterEgg();

//      shuffle meme order
        Collections.shuffle(shuffledVideos);
    }

    private void easterEgg() {
        btnRight.setOnMouseClicked(mouseEvent -> {
            System.out.println(center.getWidth());
            musicCounter++;
            switch (musicCounter) {
                case 5:
                    musicPlayer.dispose();
                    backgroundMusic("spongeBob.mp3");
                    break;
                case 12:
                    musicPlayer.dispose();
                    backgroundMusic("stadiumRave.mp3");
                    break;
                case 20:
                    musicPlayer.dispose();
                    backgroundMusic("wii.mp3");
                    break;
            }
        });

        btnLeft.setOnMouseClicked(mouseEvent -> {
//            make background music continue play after there is no meme left
            if (!musicPlayer.isAutoPlay()) {
                musicPlayer.play();
            }

//            if a meme is already running, current mediaPlayer will be removed
            if (mediaPlayer != null) {
                mediaPlayer.dispose();
                center.getChildren().remove(mediaView);
            }

            if (videoCounter < videoFiles.length) {
                String nextVideo = shuffledVideos.get(videoCounter);
                musicPlayer.stop();
                mediaPlayer = new CreateMediaPlayer().getMediaPlayer(mediaView, new File("src/main/resources/videos/" + nextVideo + ".mp4"), false);
                mediaView = new MediaView(mediaPlayer);
                center.getChildren().add(mediaView);
                mediaPlayer.setOnEndOfMedia(() -> {
                    center.getChildren().remove(mediaView);
                    musicPlayer.play();
                });
            }
            videoCounter++;
        });
    }

    private void backgroundMusic(String file) {
        Media media = new Media(new File("src/main/resources/sound/music/" + file).toURI().toString());
        musicPlayer = new MediaPlayer(media);
        musicPlayer.setCycleCount(MediaPlayer.INDEFINITE);
        musicPlayer.play();
        MusicPlayerSingleton.getInstance().setMediaPlayer(musicPlayer);
    }
}
