package de.hdm_stuttgart.battlearena.Main;

import de.hdm_stuttgart.battlearena.Model.Inputs.InputHandler;
import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Rectangle2D;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.scene.text.Font;
import javafx.stage.Screen;
import javafx.stage.Stage;

import javafx.stage.StageStyle;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.Objects;

public class Main extends Application {

    private static final Logger log = LogManager.getLogger(Main.class);
    Rectangle2D screen = Screen.getPrimary().getVisualBounds();

    InputHandler inputHandler = InputHandler.getInstance();

    public static void main(String[] args) {
        launch(args);
    }

    @Override
    public void start(Stage stage) throws Exception {
//        loading font in start() because CSS can't handle whitespace in folder names
        Font.loadFont(getClass().getResourceAsStream("/fonts/StarshipShadow.ttf"), 50);

//        TODO: revert back to intro scene
        Parent root =  FXMLLoader.load(Objects.requireNonNull(getClass().getResource("/fxml/Intro.fxml")));

        Scene scene = new Scene(root);

        scene.setOnKeyPressed(inputHandler::handleKeyPress);
        scene.setOnKeyReleased(inputHandler::handleKeyRelease);

        stage.setTitle("BattleArena");
        stage.getIcons().add(new Image("file:src/main/resources/textures/images/icon.png"));
        stage.setScene(scene);
        stage.minHeightProperty().setValue(400);
        stage.minWidthProperty().setValue(600);
        stage.setMaximized(true);
        stage.setWidth(screen.getWidth());
        stage.setHeight(screen.getHeight());
        stage.initStyle(StageStyle.UNDECORATED);
        scene.getStylesheets().add(Objects.requireNonNull(this.getClass().getResource("/styles/style.css")).toExternalForm());
        stage.setScene(scene);

        stage.show();
        log.debug("Project started successfully!");
    }
}