package de.hdm_stuttgart.battlearena.Model.DataStorage.Classes;

import de.hdm_stuttgart.battlearena.Controller.Enum.GameMode;
import de.hdm_stuttgart.battlearena.Controller.Enum.GameState;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.ENUMs.ThreadStatus;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Exceptions.ParserException;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Utilities.HashGenerator;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Utilities.Parser;
import de.hdm_stuttgart.battlearena.Model.Entity.EntityClass;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class RuntimeInfo {

    private static final Logger log = LogManager.getLogger(RuntimeInfo.class);
    private static final RuntimeInfo runtimeInfoSingleton = new RuntimeInfo();
    private String mapDataGame;
    private MapData mapGame;
    private boolean offlineMode;  //if Account-Type is online but no SQL connection: start game without stats tracking
    private GameState gameState = GameState.MENU; //Default value: MENU
    private GameMode gameMode = GameMode.LOCAL; //Default value: LOCAL

    //Default value: HUMAN
    private EntityClass playerOneClass = EntityClass.HUMAN;
    private EntityClass playerTwoClass = EntityClass.HUMAN;

    //Thread Status-Updates
    private ThreadStatus threadStartup1;
    private String threadStartup1Msg;
    private ThreadStatus threadStartup2;
    private String threadStartup2Msg;
    private ThreadStatus threadUpdateMaps;
    private String threadUpdateMapsMsg;
    private ThreadStatus threadUpdateStats;
    private String threadUpdateStatsMsg;
    private ThreadStatus threadDLCommMap;
    private String threadDLCommMapMsg;
    private ThreadStatus threadSaveCommMap;
    private String threadSaveCommMapMsg;
    private ThreadStatus threadCommMapList;
    private String threadCommMapListMsg;
    private ThreadStatus threadRstStats;
    private String threadRstStatsMsg;
    private ThreadStatus threadSaveStats;
    private String threadSaveStatsMsg;
    private ThreadStatus threadCreateAcc;
    private String threadCreateAccMsg;

    //Stats Tracking during Gameplay
    private int kills;
    private int deaths;
    private int blocksDestroyed;
    private int gameTime;
    private boolean gameWon;






    private RuntimeInfo(){};

    public static RuntimeInfo getInstance(){
        return runtimeInfoSingleton;
    }

    public MapData generateMap(String mapName, String mapData) throws ParserException {
        try {
            Parser.mapDataValid(mapData);
            Parser.mapNameValid(mapName);
            return new MapData(HashGenerator.hashAndHex(mapData), mapName, 18, 18, mapData);
        }
        catch(Exception e){
            log.error(e);
            throw new ParserException(e.getMessage());
        }
    }

    public void setGameMap(MapData mapSelected){    //Create Game Scene Dropdown list directly gives reference to selected map
        try {
            mapGame = mapSelected;
            mapDataGame = mapSelected.getMapData();
        }
        catch(Exception e){
            log.error(e);
        }
    }

    public GameState getGameState() {
        return gameState;
    }

    public void setGameState(GameState gameState) {
        this.gameState = gameState;
    }

    public GameMode getGameMode() {
        return gameMode;
    }

    public void setGameMode(GameMode gameMode) {
        this.gameMode = gameMode;
    }

    public EntityClass getPlayerOneClass() {
        return playerOneClass;
    }

    public void setPlayerOneClass(EntityClass playerOneClass) {
        this.playerOneClass = playerOneClass;
    }

    public EntityClass getPlayerTwoClass() {
        return playerTwoClass;
    }

    public void setPlayerTwoClass(EntityClass playerTwoClass) {
        this.playerTwoClass = playerTwoClass;
    }

    public String getMapDataGame() {
        return mapDataGame;
    }

    public ThreadStatus getThreadStartup1() {
        return threadStartup1;
    }

    public void setThreadStartup1(ThreadStatus threadStartup1) {
        this.threadStartup1 = threadStartup1;
    }

    public String getThreadStartup1Msg() {
        return threadStartup1Msg;
    }

    public void setThreadStartup1Msg(String threadStartup1Msg) {
        this.threadStartup1Msg = threadStartup1Msg;
    }

    public ThreadStatus getThreadStartup2() {
        return threadStartup2;
    }

    public void setThreadStartup2(ThreadStatus threadStartup2) {
        this.threadStartup2 = threadStartup2;
    }

    public String getThreadStartup2Msg() {
        return threadStartup2Msg;
    }

    public void setThreadStartup2Msg(String threadStartup2Msg) {
        this.threadStartup2Msg = threadStartup2Msg;
    }

    public ThreadStatus getThreadUpdateMaps() {
        return threadUpdateMaps;
    }

    public void setThreadUpdateMaps(ThreadStatus threadUpdateMaps) {
        this.threadUpdateMaps = threadUpdateMaps;
    }

    public String getThreadUpdateMapsMsg() {
        return threadUpdateMapsMsg;
    }

    public void setThreadUpdateMapsMsg(String threadUpdateMapsMsg) {
        this.threadUpdateMapsMsg = threadUpdateMapsMsg;
    }

    public ThreadStatus getThreadUpdateStats() {
        return threadUpdateStats;
    }

    public void setThreadUpdateStats(ThreadStatus threadUpdateStats) {
        this.threadUpdateStats = threadUpdateStats;
    }

    public String getThreadUpdateStatsMsg() {
        return threadUpdateStatsMsg;
    }

    public void setThreadUpdateStatsMsg(String threadUpdateStatsMsg) {
        this.threadUpdateStatsMsg = threadUpdateStatsMsg;
    }

    public ThreadStatus getThreadDLCommMap() {
        return threadDLCommMap;
    }

    public void setThreadDLCommMap(ThreadStatus threadDLCommMap) {
        this.threadDLCommMap = threadDLCommMap;
    }

    public String getThreadDLCommMapMsg() {
        return threadDLCommMapMsg;
    }

    public void setThreadDLCommMapMsg(String threadDLCommMapMsg) {
        this.threadDLCommMapMsg = threadDLCommMapMsg;
    }

    public ThreadStatus getThreadSaveCommMap() {
        return threadSaveCommMap;
    }

    public void setThreadSaveCommMap(ThreadStatus threadSaveCommMap) {
        this.threadSaveCommMap = threadSaveCommMap;
    }

    public String getThreadSaveCommMapMsg() {
        return threadSaveCommMapMsg;
    }

    public void setThreadSaveCommMapMsg(String threadSaveCommMapMsg) {
        this.threadSaveCommMapMsg = threadSaveCommMapMsg;
    }

    public ThreadStatus getThreadCommMapList() {
        return threadCommMapList;
    }

    public void setThreadCommMapList(ThreadStatus threadCommMapList) {
        this.threadCommMapList = threadCommMapList;
    }

    public String getThreadCommMapListMsg() {
        return threadCommMapListMsg;
    }

    public void setThreadCommMapListMsg(String threadCommMapListMsg) {
        this.threadCommMapListMsg = threadCommMapListMsg;
    }

    public ThreadStatus getThreadRstStats() {
        return threadRstStats;
    }

    public void setThreadRstStats(ThreadStatus threadRstStats) {
        this.threadRstStats = threadRstStats;
    }

    public String getThreadRstStatsMsg() {
        return threadRstStatsMsg;
    }

    public void setThreadRstStatsMsg(String threadRstStatsMsg) {
        this.threadRstStatsMsg = threadRstStatsMsg;
    }

    public ThreadStatus getThreadSaveStats() {
        return threadSaveStats;
    }

    public void setThreadSaveStats(ThreadStatus threadSaveStats) {
        this.threadSaveStats = threadSaveStats;
    }

    public String getThreadSaveStatsMsg() {
        return threadSaveStatsMsg;
    }

    public void setThreadSaveStatsMsg(String threadSaveStatsMsg) {
        this.threadSaveStatsMsg = threadSaveStatsMsg;
    }

    public ThreadStatus getThreadCreateAcc() {
        return threadCreateAcc;
    }

    public void setThreadCreateAcc(ThreadStatus threadCreateAcc) {
        this.threadCreateAcc = threadCreateAcc;
    }

    public String getThreadCreateAccMsg() {
        return threadCreateAccMsg;
    }

    public void setThreadCreateAccMsg(String threadCreateAccMsg) {
        this.threadCreateAccMsg = threadCreateAccMsg;
    }

    public int getKills() {
        return kills;
    }

    public void setKills(int kills) {
        this.kills = kills;
    }

    public int getDeaths() {
        return deaths;
    }

    public void setDeaths(int deaths) {
        this.deaths = deaths;
    }

    public int getBlocksDestroyed() {
        return blocksDestroyed;
    }

    public void setBlocksDestroyed(int blocksDestroyed) {
        this.blocksDestroyed = blocksDestroyed;
    }

    public int getGameTime() {
        return gameTime;
    }

    public void setGameTime(int gameTime) {
        this.gameTime = gameTime;
    }

    public boolean isGameWon() {
        return gameWon;
    }

    public void setGameWon(boolean gameWon) {
        this.gameWon = gameWon;
    }

}
