package de.hdm_stuttgart.battlearena.Model.Entity;

import de.hdm_stuttgart.battlearena.Controller.Enum.PlayerMode;
import de.hdm_stuttgart.battlearena.Controller.GameSceneController;
import de.hdm_stuttgart.battlearena.Model.Inputs.InputHandler;

import de.hdm_stuttgart.battlearena.Model.Map.TileManager;
import javafx.geometry.BoundingBox;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import java.util.Objects;

class Player implements IEntity {

    private static final Logger log = LogManager.getLogger(Player.class);

    private final PlayerMode PLAYER_MODE;

    InputHandler inputHandler;
    CollisionHandler collisionHandler = new CollisionHandler();

    GraphicsContext gameScene;

    GameSceneController gameSceneController;

    private BoundingBox boxCollider;

    private Image directionDownOne,
            directionDownTwo,
            directionUpOne,
            directionUpTwo,
            directionLeftOne,
            directionLeftTwo,
            directionRightOne,
            directionRightTwo;

    private Image swordDown,
            swordUp,
            swordLeft,
            swordRight;

    private final EntityClass entityClass;

    private int spriteCounter = 0;
    private int spriteNumber = 1;

    private final int playerWidth = 19;
    private final int playerHeight = 35;

    private int scaledTileSize;

    private int mapPosX;
    private int mapPosY;

    private int lastMapPosX;
    private int lastMapPosY;

    private int playerSpeed;
    private EntityDirection playerDirection;

    private int health;
    private  int maxPlayerHealth;
    private int damage;

    public Player(GraphicsContext gameScene, InputHandler inputHandler, EntityClass entityClass,
                  GameSceneController gameSceneController, PlayerMode PLAYER_MODE) {
        this.gameScene = gameScene;
        this.inputHandler = inputHandler;
        this.entityClass = entityClass;
        this.gameSceneController = gameSceneController;
        this.PLAYER_MODE = PLAYER_MODE;

        initializeEntity();
        loadEntitySprites();
    }

    @Override
    public void initializeEntity() {
        scaledTileSize = gameSceneController.getScaledTileSize();
        if (PLAYER_MODE == PlayerMode.PLAYER_ONE) {
            mapPosX = 60;
            mapPosY = 60;
        } else if (PLAYER_MODE == PlayerMode.PLAYER_TWO) {
            mapPosX = 760;
            mapPosY = 735;
        }

        initializePlayerStats();

        boxCollider = new BoundingBox(mapPosX + 15, mapPosY + 10, playerWidth, playerHeight);
        playerDirection = EntityDirection.DOWN;
    }

    //Values result from balancing
    private void initializePlayerStats() {
        if (entityClass == EntityClass.HUMAN) {
            maxPlayerHealth = 50;
            health = 50;
            damage = 5;
            playerSpeed = 5;
        } else if (entityClass == EntityClass.HIGH_BORN) {
            maxPlayerHealth = 100;
            health = 75;
            damage = 2;
            playerSpeed = 3;
        } else if (entityClass == EntityClass.LOW_BORN) {
            maxPlayerHealth = 10;
            health = 10;
            damage = 10;
            playerSpeed = 7;
        } else {
            maxPlayerHealth = 40;
            health = 15;
            damage = 7;
            playerSpeed = 5;
        }
    }

    @Override
    public void loadEntitySprites() {
        try {
            if (entityClass == EntityClass.HUMAN) {
                loadHumanSprites();
                loadWeaponSprites();
            } else if (entityClass == EntityClass.HIGH_BORN) {
                loadHighBornSprites();
                loadWeaponSprites();
            } else if (entityClass == EntityClass.LOW_BORN) {
                loadLowBornSprites();
                loadWeaponSprites();
            } else {
                loadSentinelsSprites();
                loadWeaponSprites();
            }
        } catch (Exception e) {
            log.error(e);
        }
    }

    private void loadHumanSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanDown00.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanDown01.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanUp00.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanUp01.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanLeft00.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanLeft01.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanRight00.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanRight01.png")));
    }

    private void loadHighBornSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/high_born/HighBornDown00.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/high_born/HighBornDown01.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upOne.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upTwo.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftOne.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftTwo.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightOne.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightTwo.png")));
    }

    private void loadLowBornSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/HumanDown01.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/HumanDown02.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upOne.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upTwo.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftOne.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftTwo.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightOne.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightTwo.png")));
    }

    private void loadSentinelsSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/sentinels/SentinelDown00.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/sentinels/SentinelDown01.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upOne.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upTwo.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftOne.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftTwo.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightOne.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightTwo.png")));
    }

    private void loadWeaponSprites() {
        swordUp = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordUp.png")));
        swordDown = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordDown.png")));
        swordLeft = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordLeft.png")));
        swordRight = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordRight.png")));
    }

    @Override
    public void updateEntityMovement(GameSceneController gameScene) {
        if (PLAYER_MODE == PlayerMode.PLAYER_ONE) {
            if (inputHandler.isMoveUp() || inputHandler.isMoveDown() || inputHandler.isMoveLeft() || inputHandler.isMoveRight()) {
                if (inputHandler.isMoveUp()) {
                    playerDirection = EntityDirection.UP;
                } else if (inputHandler.isMoveDown()) {
                    playerDirection = EntityDirection.DOWN;
                } else if (inputHandler.isMoveLeft()) {
                    playerDirection = EntityDirection.LEFT;
                } else if (inputHandler.isMoveRight()) {
                    playerDirection = EntityDirection.RIGHT;
                }

                boolean isWalkableTile = collisionHandler.handleMapCollision(this, scaledTileSize, gameSceneController);

                performEntityMovement(isWalkableTile);

                boolean isEntityCollision = boxCollider.intersects(gameScene.getEnemy().getBoxCollider());
                //boolean isEntityCollision = collisionHandler.handleBoxCollision(boxCollider, gameScene.getEnemy().getBoxCollider());

                performEntityCollision(isEntityCollision);

                updateEntityWalkAnimation();
            }
        } else if (PLAYER_MODE == PlayerMode.PLAYER_TWO) {
            if (inputHandler.isSdMoveUp() || inputHandler.isSdMoveDown() || inputHandler.isSdMoveLeft() || inputHandler.isSdMoveRight()) {
                if (inputHandler.isSdMoveUp()) {
                    playerDirection = EntityDirection.UP;
                } else if (inputHandler.isSdMoveDown()) {
                    playerDirection = EntityDirection.DOWN;
                } else if (inputHandler.isSdMoveLeft()) {
                    playerDirection = EntityDirection.LEFT;
                } else if (inputHandler.isSdMoveRight()) {
                    playerDirection = EntityDirection.RIGHT;
                }

                boolean isWalkableTile = collisionHandler.handleMapCollision(this, scaledTileSize, gameSceneController);

                performEntityMovement(isWalkableTile);

                boolean isEntityCollision = boxCollider.intersects(gameScene.getPlayer().getBoxCollider());
                //boolean isEntityCollision = collisionHandler.handleBoxCollision(boxCollider, gameScene.getEnemy().getBoxCollider());

                performEntityCollision(isEntityCollision);

                updateEntityWalkAnimation();
            }
        }
    }

    private void performEntityMovement(boolean isWalkableTile) {
        lastMapPosX = mapPosX;
        lastMapPosY = mapPosY;

        if (isWalkableTile) {
            switch (playerDirection) {
                case UP:
                    mapPosY -= playerSpeed;
                    break;
                case DOWN:
                    mapPosY += playerSpeed;
                    break;
                case LEFT:
                    mapPosX -= playerSpeed;
                    break;
                case RIGHT:
                    mapPosX += playerSpeed;
            }
        }
    }

    private void performEntityCollision(boolean isEntityCollision) {
        if (isEntityCollision) {
            if (playerDirection == EntityDirection.UP) {
                mapPosY = lastMapPosY + 30;
            } else if (playerDirection == EntityDirection.DOWN) {
                mapPosY = lastMapPosY - 30;
            } else if (playerDirection == EntityDirection.LEFT) {
                mapPosX = lastMapPosX + 30;
            } else {
                mapPosX = lastMapPosX - 30;
            }
        }

        boxCollider = new BoundingBox(mapPosX + 15, mapPosY + 10, playerWidth, playerHeight);
    }
    @Override
    public void checkHealTile(IEntity entity, GraphicsContext graphicsContext){
        int xTile = entity.getMapPosX() / gameSceneController.getScaledTileSize();
        int yTile = entity.getMapPosY() / gameSceneController.getScaledTileSize();
        if(TileManager.tileMap[yTile][xTile +1 ] == 5){
            entity.healPlayer(1);
            TileManager.tileMap[yTile][xTile +1] = 2;
            log.info("Healed +1: " + health);
        }
    }

    @Override
    public void attack(IEntity entity, GraphicsContext graphicsContext) {
        BoundingBox hitBox;

        int xTile = mapPosX / gameSceneController.getScaledTileSize();
        int yTile = mapPosY / gameSceneController.getScaledTileSize();
        //log.info(xTile);
        //log.info(yTile);
        //int xTile = (int) (Math.random() * (18 - 1));   //Get xTile Coordinate +1 /-1 of playerTile
        //int yTile = (int) (Math.random() * (18 - 1));   //Get yTile Coordinate +1 /-1 of playerTile
        double dropChance = 0.5;
        double randomDropChance = Math.random() * 1;
        //log.info(randomDropChance);

        //Added and subtracted numbers from variables are the pixel insets of the player sprite

        int attackRange = 30;
        int attackWidth = 10;

        if (inputHandler.isAttack() && PLAYER_MODE == PlayerMode.PLAYER_ONE ||
                inputHandler.isSdAttack() && PLAYER_MODE == PlayerMode.PLAYER_TWO) {
            if (playerDirection == EntityDirection.UP) {
                hitBox = new BoundingBox(mapPosX + playerWidth, mapPosY - 10, attackWidth, attackRange);
                graphicsContext.strokeRect(mapPosX + playerWidth, mapPosY - 10, attackWidth, attackRange);
                graphicsContext.drawImage(swordUp, mapPosX + 8, mapPosY - 10, 32, 32);
                if (hitBox.intersects(entity.getBoxCollider())) {
                    entity.gotHit(damage);
                    graphicsContext.strokeText("Hit", 10, 10);

                    if (((TileManager.tileMap[xTile][yTile] == 3) && randomDropChance > dropChance)||
                            ((TileManager.tileMap[xTile][yTile] == 1)  && randomDropChance > dropChance))
                        TileManager.tileMap[yTile-1][xTile+1] = 5;
                    //TileManager.tileMap[yTile-2][xTile] = 5;
                    //TileManager.tileMap[yTile-3][xTile] = 5;
                }
            } else if (playerDirection == EntityDirection.DOWN) {
                hitBox = new BoundingBox(mapPosX + playerWidth, mapPosY + playerHeight, attackWidth, attackRange);
                graphicsContext.strokeRect(mapPosX + playerWidth, mapPosY + playerHeight, attackWidth, attackRange);
                graphicsContext.drawImage(swordDown, mapPosX + 8, mapPosY + playerHeight, 32, 32);
                if (hitBox.intersects(entity.getBoxCollider())) {
                    entity.gotHit(damage);
                    graphicsContext.strokeText("Hit", 10, 10);

                    if (((TileManager.tileMap[xTile][yTile] == 3) && randomDropChance > dropChance)||
                            ((TileManager.tileMap[xTile][yTile] == 1)  && randomDropChance > dropChance))
                        TileManager.tileMap[yTile-1][xTile+1] = 5;
                    //TileManager.tileMap[yTile-2][xTile] = 5;
                    //TileManager.tileMap[yTile-3][xTile] = 5;
                }
            } else if (playerDirection == EntityDirection.LEFT) {
                hitBox = new BoundingBox(mapPosX - attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.strokeRect(mapPosX - attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.drawImage(swordLeft, mapPosX - 8, mapPosY + 8, 32, 32);
                if (hitBox.intersects(entity.getBoxCollider())) {
                    entity.gotHit(damage);
                    graphicsContext.strokeText("Hit", 10, 10);

                    if (((TileManager.tileMap[xTile][yTile] == 3) && randomDropChance > dropChance)||
                            ((TileManager.tileMap[xTile][yTile] == 1)  && randomDropChance > dropChance))
                        TileManager.tileMap[yTile-1][xTile+1] = 5;
                    //TileManager.tileMap[yTile-2][xTile] = 5;
                    //TileManager.tileMap[yTile-3][xTile] = 5;
                }
            } else {
                hitBox = new BoundingBox(mapPosX + playerWidth + attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.strokeRect(mapPosX + playerWidth + attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.drawImage(swordRight, mapPosX + playerWidth + 8, mapPosY + 8, 32, 32);
                if (hitBox.intersects(entity.getBoxCollider())) {
                    entity.gotHit(damage);
                    graphicsContext.strokeText("Hit", 10, 10);

                    if (((TileManager.tileMap[xTile][yTile] == 3) && randomDropChance > dropChance)||
                            ((TileManager.tileMap[xTile][yTile] == 1)  && randomDropChance > dropChance))
                        TileManager.tileMap[yTile-1][xTile+1] = 5;
                    //TileManager.tileMap[yTile-2][xTile] = 5;
                    //TileManager.tileMap[yTile-3][xTile] = 5;
                }
            }
        }
    }

    @Override
    public void updateEntityWalkAnimation() {
        spriteCounter++;

        if (spriteCounter > 10) {
            if (spriteNumber == 1) {
                spriteNumber = 2;
            } else if (spriteNumber == 2) {
                spriteNumber = 1;
            }
            spriteCounter = 0;
        }
    }

    @Override
    public void renderEntity(GraphicsContext graphicsContext) {
        Image playerSprite = null;

        //for debugging collision
        graphicsContext.strokeRect(mapPosX + 15, mapPosY + 10, playerWidth, playerHeight);

        switch (playerDirection) {
            case UP:
                if (spriteNumber == 1) {
                    playerSprite = directionUpOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionUpTwo;
                }
                break;
            case DOWN:
                if (spriteNumber == 1) {
                    playerSprite = directionDownOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionDownTwo;
                }
                break;
            case LEFT:
                if (spriteNumber == 1) {
                    playerSprite = directionLeftOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionLeftTwo;
                }
                break;
            case RIGHT:
                if (spriteNumber == 1) {
                    playerSprite = directionRightOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionRightTwo;
                }
                break;
        }

        graphicsContext.drawImage(playerSprite, mapPosX, mapPosY, 48, 48);
    }

    @Override
    public void gotHit(int damageDone) {
        health -= damageDone;
    }
    @Override
    public void healPlayer(int healthRegenerated) {
        int regeneratedHealth = health + healthRegenerated;

        if (regeneratedHealth < maxPlayerHealth) {
            health += healthRegenerated;
        } else {
            health = maxPlayerHealth;
        }
        log.info(health);
    }

    @Override
    public BoundingBox getBoxCollider() {
        return boxCollider;
    }

    @Override
    public EntityDirection getEntityDirection() {
        return playerDirection;
    }

    @Override
    public int getEntitySpeed() {
        return playerSpeed;
    }

    @Override
    public int getMapPosX() {
        return mapPosX;
    }

    @Override
    public int getMapPosY() {
        return mapPosY;
    }

}